# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
module Vulnerability
  # Manage the vulnerability config data
  module Config
    def grype_root
      kernel = Facter.value(:kernel)
      case kernel
      when 'Linux'
        '/usr/local'
      when 'windows'
        'c:/vulnerability'
      else
        fail "'#{kernel}' not (yet) supported."
      end
    end

    def grype_binary
      kernel = Facter.value(:kernel)
      case kernel
      when 'Linux'
        '/usr/local/bin/grype'
      when 'windows'
        'C:\vulnerability\bin\grype.exe'
      else
        fail "'#{kernel}' not (yet) supported."
      end
    end

    def config_data
      config_text = File.read("#{grype_root}/etc/vulnerability.conf")
      config_data = config_text.match(/directories = (?<directories>.*)excludes = (?<excludes>.*)level = (?<level>.*)/m)
      directories = config_data['directories'].strip.split(',')
      excludes    = config_data['excludes'].strip.split(',')
      level       = config_data['level'].strip
      # Return the config data as a Hash
      { :level => level, :directories => directories, :excludes => excludes }
    end
  end
end
