# frozen_string_literal: true

require 'puppet_x/vulnerability/selection'
#
# See the file "LICENSE" for the full license governing this code.
#
module Vulnerability
  # Implementation of puppet function vulnerability::validate
  module Validate
    include ::Vulnerability::Selection

    def self.included(base)
      base.dispatch :validate do
        param       'Vulnerability::Level',     :level
        param       'Array[String[1]]',         :allow_list
        param       'Boolean',                  :fail_on_unsafe
        return_type 'Hash'
      end
    end

    def validate(level, allow_list, fail_on_unsafe)
      return {} unless closure_scope.include?('cve_list')

      selected_cves = level_selection(closure_scope['cve_list'], level)
      selected_cves = filter_cves(selected_cves, allow_list)
      raise(Puppet::Error, "System contains vulnerabilies above level '#{level}', that are not on allow list") if !selected_cves.empty? && fail_on_unsafe

      selected_cves
    end
  end
end
