# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
class Puppet::Parameter::StatusProperty < Puppet::Property
  include EasyType
  include EasyType::Validators::Integer

  data_type('Optional[Integer]')

  desc "The maximum number of #{name} vulnerabilities you allow on your system."

  to_translate_to_resource do |raw_resource|
    raw_resource[name]
  end

  on_create do
    nil
  end

  on_modify do
    fqdn = Facter.value(:fqdn)
    message = "More #{name} vulnerabilities found than specified value #{value} on #{fqdn}; actual found is: #{provider.send(name)}."
    raise Puppet::Error, message if /error/i.match?(resource.when_more)

    Puppet.warning message
  end

  # rubocop: disable Naming/MethodParameterName
  def insync?(is)
    if resource.name != Facter.value('fqdn')
      Puppet.debug "#{path}: Skipping resource with name '#{resource.name}'"
      return true
    end
    return true if is <= value

    super
  end
  # rubocop: enable Naming/MethodParameterName
end
