#!/opt/puppetlabs/puppet/bin/ruby
# frozen_string_literal: true

#
#
# See the file "LICENSE" for the full license governing this code.
#
#
require 'json'
require 'open3'
require 'puppet'
require 'deep_merge'
require 'facter'
require 'yaml'

#
# Task to fetch vulnerability data from the configured directories.
# This task is used in conjuction with InControl.
#
class FetchVulnerabilitiesTask
  def self.run
    instance = new
    instance.run
  end

  def run
    fetch_input
    execute
  ensure
    report
  end

  def fetch_input
    params = JSON.parse($stdin.read)
    if params.key?('_installdir')
      #
      # Use loadpath specified in the params
      #
      $LOAD_PATH << "#{params['_installdir']}/vulnerability/lib"
    else
      begin
        Puppet.initialize_settings
      rescue Puppet::DevError
        # When run in Rspec, this is called multiple times
      end
    end
    require 'puppet_x/vulnerability/config'
    require 'puppet_x/vulnerability/selection'

    FetchVulnerabilitiesTask.include Vulnerability::Selection
    FetchVulnerabilitiesTask.include Vulnerability::Config

    @result = {}
  end

  def report
    puts @result.to_json
  end

  def execute
    directories = config_data[:directories]
    level       = config_data[:level]
    config      = "#{grype_root}/etc/grype.yaml"
    template    = "#{grype_root}/etc/grype_yaml.tpl"

    directories.each do |directory|
      vulnerabilities = `#{grype_binary} -c #{config} dir:#{directory} -o template -t #{template} -q`
      data = YAML.safe_load(vulnerabilities)&.reduce({}, :deep_merge) unless vulnerabilities.empty?
      data ||= {}
      @result = @result.deep_merge(data)
    end
    @result = level_selection(@result, level)
  end
end

FetchVulnerabilitiesTask.run if __FILE__ == $PROGRAM_NAME
